@@ -1,4 +1,3 @@
-
 package DBD::Mock;
 
 sub import {
@@ -7,7 +6,7 @@ sub import {
 }
 
 # --------------------------------------------------------------------------- #
-#   Copyright (c) 2004 Stevan Little, Chris Winters 
+#   Copyright (c) 2004-2007 Stevan Little, Chris Winters 
 #   (spawned from original code Copyright (c) 1994 Tim Bunce)
 # --------------------------------------------------------------------------- #
 #   You may distribute under the terms of either the GNU General Public
@@ -19,7 +18,7 @@ use warnings;
 
 require DBI;
 
-our $VERSION = '1.32';
+our $VERSION = '1.35';
 
 our $drh    = undef;    # will hold driver handle
 our $err    = 0;        # will hold any error codes
@@ -109,6 +108,13 @@ sub connect {
         $attributes->{mock_attribute_aliases} = DBD::Mock::_get_mock_attribute_aliases($dbname);
         $attributes->{mock_database_name} = $dbname;
     }
+
+    # Need to protect AutoCommit from $dbh caching - RobK.
+    my $autocommit = 1;
+    if( exists $attributes->{ 'AutoCommit' } ) {
+        $autocommit = delete $attributes->{ 'AutoCommit' };
+    }
+
     my $dbh = DBI::_new_dbh($drh, {
         Name                   => $dbname,       
         # holds statement parsing coderefs/objects
@@ -117,9 +123,16 @@ sub connect {
         mock_statement_history => [],
         # ability to fake a failed DB connection
         mock_can_connect       => 1,
+        # ability to make other things fail :)
+        mock_can_prepare       => 1,
+        mock_can_execute       => 1,
+        mock_can_fetch         => 1,        
         # rest of attributes
         %{ $attributes },
     }) || return;
+
+    $dbh->STORE( 'AutoCommit' => $autocommit );
+
     return $dbh;
 }
 
@@ -203,6 +216,17 @@ sub get_info {
 
 sub prepare {
     my($dbh, $statement) = @_;
+    
+    unless ($dbh->{mock_can_connect}) {
+        $dbh->DBI::set_err(1, "No connection present");
+        return;
+    }
+    unless ($dbh->{mock_can_prepare}) {
+        $dbh->DBI::set_err(1, "Cannot prepare");
+        return;
+    }
+    $dbh->{mock_can_prepare}++ if $dbh->{mock_can_prepare} < 0;
+    
 
     eval {
         foreach my $parser ( @{ $dbh->{mock_parser} } ) {
@@ -377,11 +401,14 @@ sub selectcol_arrayref {
     return [ map { $_->[0] } @{$a_ref} ]
 }
 
+{
+    my %autocommit;
 sub FETCH {
     my ( $dbh, $attrib ) = @_;
     $dbh->trace_msg( "Fetching DB attrib '$attrib'\n" );
     if ($attrib eq 'AutoCommit') {
-        return $dbh->{AutoCommit};
+        $dbh->trace_msg( "Fetching AutoCommit\n" );
+        return $autocommit{$dbh};
     }
      elsif ($attrib eq 'Active') {
         return $dbh->{mock_can_connect};
@@ -420,7 +447,7 @@ sub STORE {
     my ( $dbh, $attrib, $value ) = @_;   
     $dbh->trace_msg( "Storing DB attribute '$attrib' with '" . (defined($value) ? $value : 'undef') . "'\n" );
     if ($attrib eq 'AutoCommit') {
-        $dbh->{AutoCommit} = $value;
+        $autocommit{$dbh} = $value;
         return $value;
     }
     elsif ( $attrib eq 'mock_clear_history' ) {
@@ -512,6 +539,7 @@ sub STORE {
       return $dbh->{$attrib} = $value;
   }
 }
+}
 
 sub DESTROY {
     undef
@@ -555,6 +583,11 @@ sub execute {
         $dbh->DBI::set_err(1, "No connection present");
         return 0;
     }
+    unless ($dbh->{mock_can_execute}) {
+        $dbh->DBI::set_err(1, "Cannot execute");
+        return 0;
+    }
+    $dbh->{mock_can_execute}++ if $dbh->{mock_can_execute} < 0;    
 
     my $tracker = $sth->FETCH( 'mock_my_history' );
     
@@ -570,6 +603,10 @@ sub execute {
     if (my $session = $dbh->{mock_session}) {
         eval {
             $session->verify_bound_params($dbh, $tracker->bound_params());
+            my $idx = $session->{state_index} - 1;
+            my @results = @{$session->{states}->[$idx]->{results}};
+            shift @results;
+            $tracker->{return_data} = \@results;            
         };
         if ($@) {
             my $session_error = $@;
@@ -599,10 +636,16 @@ sub execute {
 
 sub fetch {
     my ($sth) = @_;
-    unless ($sth->{Database}->{mock_can_connect}) {
-        $sth->{Database}->DBI::set_err(1, "No connection present");
+    my $dbh = $sth->{Database};
+    unless ($dbh->{mock_can_connect}) {
+        $dbh->DBI::set_err(1, "No connection present");
         return;
     }
+    unless ($dbh->{mock_can_fetch}) {
+        $dbh->DBI::set_err(1, "Cannot fetch");
+        return;
+    }
+    $dbh->{mock_can_fetch}++ if $dbh->{mock_can_fetch} < 0;
     
     my $tracker = $sth->FETCH( 'mock_my_history' );
     return $tracker->next_record;
@@ -622,12 +665,18 @@ sub fetchrow_arrayref {
 
 sub fetchrow_hashref {
     my ($sth, $name) = @_;
+    my $dbh = $sth->{Database};
     # handle any errors since we are grabbing
     # from the tracker directly
-    unless ($sth->{Database}->{mock_can_connect}) {
-        $sth->{Database}->DBI::set_err(1, "No connection present");
+    unless ($dbh->{mock_can_connect}) {
+        $dbh->DBI::set_err(1, "No connection present");
         return;
     }    
+    unless ($dbh->{mock_can_fetch}) {
+        $dbh->DBI::set_err(1, "Cannot fetch");
+        return;
+    }
+    $dbh->{mock_can_fetch}++ if $dbh->{mock_can_fetch} < 0;    
     
     # first handle the $name, it will default to NAME
     $name ||= 'NAME';
@@ -651,12 +700,18 @@ sub fetchrow_hashref {
 
 sub fetchall_hashref {
     my ($sth, $keyfield) = @_;
+    my $dbh = $sth->{Database};
     # handle any errors since we are grabbing
     # from the tracker directly
-    unless ($sth->{Database}->{mock_can_connect}) {
-        $sth->{Database}->DBI::set_err(1, "No connection present");
+    unless ($dbh->{mock_can_connect}) {
+        $dbh->DBI::set_err(1, "No connection present");
         return;
     }      
+    unless ($dbh->{mock_can_fetch}) {
+        $dbh->DBI::set_err(1, "Cannot fetch");
+        return;
+    }
+    $dbh->{mock_can_fetch}++ if $dbh->{mock_can_fetch} < 0;      
 
     my $tracker = $sth->FETCH( 'mock_my_history' );
     my $rethash = {};
@@ -679,7 +734,7 @@ sub fetchall_hashref {
             }
         }
         unless ($found) {
-            $sth->{Database}->DBI::set_err(1, "Could not find key field '$keyfield'");
+            $dbh->DBI::set_err(1, "Could not find key field '$keyfield'");
             return;
         }
     }
@@ -778,6 +833,9 @@ sub FETCH {
     elsif ( $attrib eq 'mock_params' ) {
         return $tracker->bound_params;
     }
+    elsif ( $attrib eq 'mock_records' ) {
+        return $tracker->return_data;
+    }    
     elsif ( $attrib eq 'mock_num_records' || $attrib eq 'mock_num_rows' ) {
         return $tracker->num_rows;
     }    
@@ -1554,6 +1612,14 @@ These properties will dispatch to the Driver's properties of the same name.
 
 =head2 Database Driver Methods
 
+=over 4
+
+=item B<last_insert_id>
+
+This returns the value of C<mock_last_insert_id>.
+
+=back
+
 In order to capture begin_work(), commit(), and rollback(), DBD::Mock will create statements for them, as if you had issued them in the appropriate SQL command line program. They will go through the standard prepare()-execute() cycle, meaning that any custom SQL parsers will be triggered and DBD::Mock::Session will need to know about these statements.
 
 =over 4
@@ -1572,6 +1638,7 @@ This will create a statement with SQL of "ROLLBACK" and no parameters.
 
 =back
 
+
 =head2 Statement Handle Properties
 
 =over 4
@@ -1895,15 +1962,14 @@ I would also like to add the ability to bind a subroutine (or possibly an object
 
 =head1 CODE COVERAGE
 
-I use L<Devel::Cover> to test the code coverage of my tests, below is the L<Devel::Cover> report on this module test suite.
+We use L<Devel::Cover> to test the code coverage of my tests, below is the L<Devel::Cover> report on this module test suite.
 
- ---------------------------- ------ ------ ------ ------ ------ ------ ------
- File                           stmt   bran   cond    sub    pod   time  total
- ---------------------------- ------ ------ ------ ------ ------ ------ ------
- DBD/Mock.pm                    90.9   85.5   76.0   94.1    0.0  100.0   88.4
- ---------------------------- ------ ------ ------ ------ ------ ------ ------
- Total                          90.9   85.5   76.0   94.1    0.0  100.0   88.4
- ---------------------------- ------ ------ ------ ------ ------ ------ ------
+  ---------------------------- ------ ------ ------ ------ ------ ------ ------
+  File                           stmt   bran   cond    sub    pod   time  total
+  ---------------------------- ------ ------ ------ ------ ------ ------ ------
+  blib/lib/DBD/Mock.pm           92.0   86.6   77.9   95.3    0.0  100.0   89.5
+  Total                          92.0   86.6   77.9   95.3    0.0  100.0   89.5
+  ---------------------------- ------ ------ ------ ------ ------ ------ ------
 
 =head1 SEE ALSO
 
@@ -1927,6 +1993,10 @@ L<http://groups-beta.google.com/group/DBDMock>
 
 =over 4
 
+=item Thanks to Ryan Gerry for his patch in RT #26604
+
+=item Thanks to Marc Beyer for his patch in RT #16951
+
 =item Thanks to Justin DeVuyst for the mock_connect_fail idea
 
 =item Thanks to Thilo Planz for the code for C<bind_param_inout>
@@ -1939,11 +2009,15 @@ L<http://groups-beta.google.com/group/DBDMock>
 
 =item Thanks to Andrew W. Gibbs for the C<mock_last_insert_ids> patch and test
 
+=item Thanks to Chas Owens for patch and test for the C<mock_can_prepare>, C<mock_can_execute>, and C<mock_can_fetch> features.
+
 =back
 
 =head1 COPYRIGHT
 
-Copyright (c) 2004 & 2005 Stevan Little, Chris Winters. All rights reserved.
+Copyright (C) 2004 Chris Winters <chris@cwinters.com>
+
+Copyright (C) 2004-2007 Stevan Little <stevan@iinteractive.com>
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
